﻿using Infragistics.Controls.Charts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace XamDataChartColumnSeriesBinderCase
{
    public class SeriesBindingInfo: FrameworkElement
    {
        public static DependencyProperty OwnerProperty =
          DependencyProperty.Register("Owner",
          typeof(XamDataChart), typeof(SeriesBindingInfo),
          new PropertyMetadata(null,
              (o, e) =>
              {
                  (o as SeriesBindingInfo).OnOwnerChanged(e);
              }));

        public XamDataChart Owner
        {
            get
            {
                return (XamDataChart)GetValue(OwnerProperty);
            }
            set
            {
                SetValue(OwnerProperty, value);
            }
        }

        public static DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register("ItemsSource",
            typeof(IEnumerable), typeof(SeriesBindingInfo),
            new PropertyMetadata(null,
                (o, e) =>
                {
                    (o as SeriesBindingInfo).OnItemsSourceChanged(e);
                }));

        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        public static DependencyProperty SeriesTemplateProperty =
            DependencyProperty.Register("SeriesTemplate",
            typeof(DataTemplate), typeof(SeriesBindingInfo),
            new PropertyMetadata(null,
                (o, e) =>
                {
                    (o as SeriesBindingInfo).OnSeriesTemplateChanged(e);
                }
                    ));

        public DataTemplate SeriesTemplate
        {
            get
            {
                return (DataTemplate)GetValue(SeriesTemplateProperty);
            }
            set
            {
                SetValue(SeriesTemplateProperty, value);
            }
        }

        protected void OnItemsSourceChanged(
            DependencyPropertyChangedEventArgs e)
        {
            Clear();

            if (e.OldValue != null)
            {
                UnRegCollection(e.OldValue as IEnumerable);
            }
            if (e.NewValue != null)
            {
                RegCollection(e.NewValue as IEnumerable);
                UpdateFromCollection(e.NewValue as IEnumerable);

                if (this.Owner != null)
                {
                    bool hasCrosshairLayer = false;
                    bool hasTooltipLayer = false;

                    foreach(var x in this.Owner.Series)
                    {
                        if(x is CrosshairLayer)
                        {
                            hasCrosshairLayer = true;
                        }
                        if(x is CategoryToolTipLayer)
                        {
                            hasTooltipLayer = true;
                        }
                    }

                    if (!hasCrosshairLayer)
                    {
                        Owner.Series.Add(new CrosshairLayer());
                    }
                    if (!hasTooltipLayer)
                    {
                        Owner.Series.Add(new CategoryToolTipLayer());
                    }
                }
            }
        }

        public static DependencyProperty OuterContextProperty =
        DependencyProperty.Register("OuterContext",
        typeof(object), typeof(SeriesBindingInfo),
        new PropertyMetadata(null,
            (o, e) =>
            {
            }));

        public object OuterContext
        {
            get
            {
                return GetValue(OuterContextProperty);
            }
            set
            {
                SetValue(OuterContextProperty, value);
            }
        }

        protected void OnSeriesTemplateChanged(
            DependencyPropertyChangedEventArgs e)
        {
            Clear();
            UpdateFromCollection(ItemsSource);
        }

        private void OnOwnerChanged(
            DependencyPropertyChangedEventArgs e)
        {
            SetBinding(DataContextProperty,
            new Binding("OuterContext")
            {
                Source = this
            });
            Clear();
            UpdateFromCollection(ItemsSource);
        }

        private void Clear()
        {
            if (Owner == null)
            {
                return;
            }
            Owner.Series.Clear();
        }

        private void UpdateFromCollection(
            IEnumerable collection)
        {
            if (collection == null)
            {
                return;
            }
            foreach (object item in collection)
            {
                AddItem(item);
            }
        }

        private void AddItem(
            object item)
        {
            if (SeriesTemplate == null)
            {
                return;
            }

            Series series = CreateSeries(item);

            if (series != null)
            {
                Owner.Series.Add(series);
            }
        }

        private void InsertItem(
            object item,
            int index)
        {
            if (SeriesTemplate == null)
            {
                return;
            }

            Series series = CreateSeries(item);

            if (series != null)
            {
                Owner.Series.Insert(index, series);
            }
        }

        private void RemoveItem(
           object item)
        {
            if (SeriesTemplate == null)
            {
                return;
            }

            for (var i = 0; i < Owner.Series.Count; i++)
            {
                var context = Owner.Series[i].DataContext as SeriesDataContext;

                if (context != null && context.Item == item)
                {
                    Owner.Series.RemoveAt(i);
                    break;
                }
            }
        }

        private Series CreateSeries(object item)
        {
            Series series = SeriesTemplate.LoadContent() as Series;
            if (series != null)
            {
                SeriesDataContext context =
                    new SeriesDataContext()
                    {
                        Item = item,
                        Owner = Owner
                    };
                BindingOperations.SetBinding(
                    context,
                    SeriesDataContext.OuterContextProperty,
                    new Binding("OuterContext")
                    {
                        Source = this
                    });

                series.DataContext = context;
                Axis xAxis = null;
                Axis yAxis = null;
                GetAxes(series, ref xAxis, ref yAxis);

                if (xAxis != null)
                {
                    UpdateAxis(context, xAxis);
                }
                if (yAxis != null)
                {
                    UpdateAxis(context, yAxis);
                }
            }
            return series;
        }

        private void UpdateAxis(SeriesDataContext context, Axis axis)
        {
            axis.DataContext = context;
            if (!Owner.Axes.Contains(axis))
            {
                Owner.Axes.Add(axis);
            }
        }

        private void GetAxes(Series series, ref Axis xAxis, ref Axis yAxis)
        {
            if (series is HorizontalAnchoredCategorySeries)
            {
                xAxis = (series as HorizontalAnchoredCategorySeries).XAxis;
                yAxis = (series as HorizontalAnchoredCategorySeries).YAxis;
            }
            if (series is VerticalAnchoredCategorySeries)
            {
                xAxis = (series as VerticalAnchoredCategorySeries).XAxis;
                yAxis = (series as VerticalAnchoredCategorySeries).YAxis;
            }
            else if (series is HorizontalRangeCategorySeries)
            {
                xAxis = (series as HorizontalRangeCategorySeries).XAxis;
                yAxis = (series as HorizontalRangeCategorySeries).YAxis;
            }
            else if (series is FinancialSeries)
            {
                xAxis = (series as FinancialSeries).XAxis;
                yAxis = (series as FinancialSeries).YAxis;
            }
            else if (series is ScatterBase)
            {
                xAxis = (series as ScatterBase).XAxis;
                yAxis = (series as ScatterBase).YAxis;
            }
        }

        private void RegCollection(
            IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged
                    += SeriesBindingInfo_CollectionChanged;
            }
        }

        private void UnRegCollection(
            IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged
                    -= SeriesBindingInfo_CollectionChanged;
            }
        }

        void SeriesBindingInfo_CollectionChanged(
            object sender,
            NotifyCollectionChangedEventArgs e)
        {
            if (Owner == null)
            {
                return;
            }

            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (var item in e.NewItems)
                    {
                        AddItem(item);
                    }
                    break;
                case NotifyCollectionChangedAction.Move:
                    foreach (var item in e.OldItems)
                    {
                        RemoveItem(item);
                    }
                    var startingIndex = e.NewStartingIndex;
                    foreach (var item in e.NewItems)
                    {
                        InsertItem(item, startingIndex++);
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (var item in e.OldItems)
                    {
                        RemoveItem(item);
                    }
                    break;
                case NotifyCollectionChangedAction.Replace:
                    foreach (var item in e.OldItems)
                    {
                        RemoveItem(item);
                    }
                    var startingIndex2 = e.NewStartingIndex;
                    foreach (var item in e.NewItems)
                    {
                        InsertItem(item, startingIndex2++);
                    }
                    break;
                case NotifyCollectionChangedAction.Reset:
                    Clear();
                    UpdateFromCollection(ItemsSource);
                    break;
            }
        }
    }
}
